#!/usr/bin/perl

##########################################################
#  Physical Activity Derivation Program
#  W.Rayner & Aliki-Eleni Farmaki 
#  Version 1.0 2013-2015
#
#  Requires: 
#	Current_weight
#	Days_working
#	Hours_per_working_day	
#	Mins_sitting
#	Mins_standing
#	Mins_moving
#	Hours_transfering_weight
#	Mins_per_day_transport_work
#	Mins_per_day_walk_work
#	Mins_per_day_driving_work
#	Hours_per_day_sleeping
#	Hours_per_day_TV_video
#	Hours_per_week_mild
#	Hours_per_week_hard
#	Hours_per_week_reading_PC
#	Hours_per_week_dancing
#	Hours_per_week_leisure_sitting_standing
#	Hours_per_week_leisure_walking
#	Hours_per_week_activity1
#	Hours_per_week_activity2
#	Hours_per_week_activity3
#	Met_activity1 
#	Met_activity2 
#	Met_activity3
#
#########################################################

use strict;
use Getopt::Long;

print "   Physical Activity Derivation Program\n";
print "   William Rayner & Aliki-Eleni Farmaki\n";
print "           Version 1.0 Jan 2015\n\n";

my $file;
my $outfile = 'PAD.out';
my $logfile = 'PAD.log';
my $inifile = 'PAD.ini';

GetOptions
 (
 "i|ini=s"          => \$inifile,
 "f|file=s"         => \$file,
 "o|out=s"          => \$outfile,
 "l|log=s"          => \$logfile
 );

if (!$file)
 {
 print "No input file given\n";
 print "Usage ";
 exit(1);
 }

print "Options Set:\n";
print " Ini file:      $inifile\n";
print " Input file:    $file\n";
print " Output file:   $outfile\n";
print " Log file:      $logfile\n";

print "\n\n";

#get header allocation from PAD.ini file
print "Reading Headers\n";
my @expected_headers = read_ini($inifile);
print "Done\n";

#get met allocation for each header
print "Getting MET allocation\n";
my %met = get_met($inifile);
print "Done\n";

#to do
#check all columns are assigned that need to be
#check the minutes/hours/days/weeks assignments are ok


#get header line from file
print "Reading data headers\n";
my @dataHeader = get_data_headers($file);
print "Done; got $#dataHeader headers\n";

#extract data from the file
print "Extracting data\n";
my %data = extract_data($file);
my @samples = keys %data;
print "Done; got $#samples Samples\n";

#to do
#validate the metrics against the minutes/hours/days/weeks assignment such that hours/day !>  24, etc.
#report any errors or issues with the data


open OUT, ">$outfile" or die $!;
print OUT "SAMPLE\tKcal Work\tKcal Transport\tKcal Physical activity\tKcal at Home\tKcal Leisure\tKcal Other\tKcal Leisure Total\tKcal Remaining per week\tKcal Total Week\tKcal Total Day\tRMR\tPAL\n";

#calculate the metrics
print "Calculating Metrics\n";
foreach my $sample (@samples)
 {
 my $days = $data{$sample}{'Days_working'};
 my $weight = $data{$sample}{'Current_weight'};
 if ($weight)
  {
  my $sitting = (($met{'Mins_sitting'} * $weight)/60) * $data{$sample}{'Mins_sitting'} * $days;
  my $standing = (($met{'Mins_standing'} * $weight)/60) * $data{$sample}{'Mins_standing'} * $days;
  my $moving = (($met{'Mins_moving'} * $weight)/60) * $data{$sample}{'Mins_moving'} * $days;
  my $weighttransfer = (($met{'Mins_transfering_weight'} * $weight)/60) * $data{$sample}{'Mins_transfering_weight'} * $days;
   
  my $kcalwork = $sitting + $standing + $moving + $weighttransfer;
  
  my $walk = ($met{'Mins_per_day_walk_work'} * $weight/60) * $data{$sample}{'Mins_per_day_walk_work'} * $days;
  my $drive = ($met{'Mins_per_day_driving_work'} * $weight/60) * $data{$sample}{'Mins_per_day_driving_work'} * $days;
  
  my $kcaltransport = $walk + $drive;
  
  my $kcalphysactivity = $kcalwork + $kcaltransport;
 
  my $sleeping = $met{'Hours_per_day_sleeping'} * $weight * $data{$sample}{'Hours_per_day_sleeping'} * 7;
  my $tv = $met{'Hours_per_day_TV_video'} * $weight * $data{$sample}{'Hours_per_day_TV_video'} * 7; 
  my $mild = $met{'Hours_per_week_mild'} * $weight * $data{$sample}{'Hours_per_week_mild'};
  my $hard = $met{'Hours_per_week_hard'} * $weight * $data{$sample}{'Hours_per_week_hard'};
  my $reading = $met{'Hours_per_week_reading_PC'} * $weight * $data{$sample}{'Hours_per_week_reading_PC'};
  
  my $kcalathome = $sleeping + $tv + $mild + $hard + $reading;
   
  my $dancing = $met{'Hours_per_week_dancing'} * $weight * $data{$sample}{'Hours_per_week_dancing'};
  my $leisure = $met{'Hours_per_week_leisure_sitting_standing'} * $weight * $data{$sample}{'Hours_per_week_leisure_sitting_standing'};
  my $leisurewalking = $met{'Hours_per_week_leisure_walking'} * $weight * $data{$sample}{'Hours_per_week_leisure_walking'};
 
  my $kcalleisure = $dancing + $leisure + $leisurewalking;
 
  my $activity1 = $data{$sample}{'Met_activity1'} * $weight * $data{$sample}{'Hours_per_week_activity1'};
  my $activity2 = $data{$sample}{'Met_activity2'} * $weight * $data{$sample}{'Hours_per_week_activity2'};
  my $activity3 = $data{$sample}{'Met_activity3'} * $weight * $data{$sample}{'Hours_per_week_activity3'};
  
  my $kcalother = $activity1 + $activity2 + $activity3;
  
  my $kcalleisuretotal = $kcalleisure + $kcalother;
   
  #total all to hours used per week
  my $total1 = (($data{$sample}{'Mins_sitting'} + $data{$sample}{'Mins_standing'} + $data{$sample}{'Mins_moving'} + $data{$sample}{'Mins_transfering_weight'})/60) * $days;
  my $total2 = (($data{$sample}{'Mins_per_day_walk_work'} + $data{$sample}{'Mins_per_day_driving_work'})/60)* $days;
  my $total3 = ($data{$sample}{'Hours_per_day_sleeping'} + $data{$sample}{'Hours_per_day_TV_video'}) * 7;
  my $total4 = $data{$sample}{'Hours_per_week_mild'} + $data{$sample}{'Hours_per_week_hard'} + $data{$sample}{'Hours_per_week_reading_PC'};
  my $total5 = $data{$sample}{'Hours_per_week_dancing'} + $data{$sample}{'Hours_per_week_leisure_sitting_standing'} + $data{$sample}{'Hours_per_week_leisure_walking'};
  my $total6 = $data{$sample}{'Hours_per_week_activity1'} + $data{$sample}{'Hours_per_week_activity2'} + $data{$sample}{'Hours_per_week_activity3'};
   
  #total hours in a week
  my $total = $total1 + $total2 + $total3 + $total4 + $total5 + $total6;
  my $remaining = (24*7) - ($total1 + $total2 + $total3 + $total4 + $total5 + $total6);
  
  if ($remaining < 0)
   {
   print "WARNING: $sample hours per week recorded > 168 ($remaining)\n";
   $remaining = 0;
   }
   
  #print "$sample\t$total1\t$total2\t$total3\t$total4\t$total5\t$total6\t$total\t$remaining\n";

  my $totalweek = $remaining * $weight;
 
  my $totalkcalweek = $kcalphysactivity + $kcalathome + $kcalleisuretotal + $kcalother + $totalweek;
  my $totalkcalday = $totalkcalweek/7; 

  my $rmr = $weight * 24;
  #print "$totalkcalday/$rmr\n";
  my $pal = 0;
  
  if ($totalkcalday)
   {
   $pal = $totalkcalday/$rmr;
   }
  else
   {
   #print "WARNING: $sample has 0 total kcal per day\n";
   $pal = 0;
   }
   
  #write out the metrics
  print OUT "$sample\t$kcalwork\t$kcaltransport\t$kcalphysactivity\t$kcalathome\t$kcalleisure\t$kcalother\t$kcalleisuretotal\t$totalweek\t$totalkcalweek\t$totalkcalday\t$rmr\t$pal\n";
  }
 else
  {
  print "WARNING: $sample has no weight\n";
  }
 }




###########################################################
sub read_ini($)
 {
 my $file = $_[0];
 my @header;
 my $i = 0;
 
 open IN, "$file" or die $!;
 while (<IN>)
  {
  chomp;
  my @temp = split/\t/;
  $header[$i] = $temp[1]; 
  $i++;
  }
 return @header;
 }

sub get_met($)
 {
 my $file = $_[0];
 my %met;
 my $i = 0;
  
 open IN, "$file" or die $!;
 while (<IN>)
  {
  chomp;
  my @temp = split/\t/;
  $met{$temp[1]} = $temp[4]; 
  }
 return %met;
 }

sub get_headers($)
 {
 my $file = $_[0];
 my @header;
 my $i = 0;
 
 open IN, "$file" or die $!;
 while (<IN>)
  {
  chomp;
  my @temp = split/\t/;
  $header[$i] = $temp[0];
  $i++;
  }
 close IN; 
 return @header; 
 }
 
sub get_data_headers($)
 {
 my $file = $_[0];
 my @header;
 
 open IN, "$file" or die $!;
 my $title = <IN>;
 close IN;
 chomp $title;
 @header = split(/\t/, $title);
 return @header; 
 }
 
 
sub extract_data($)
 {
 my $file = $_[0];
 my %data;
 
 open IN, "$file" or die $!;
 my $titleline = <IN>;
 chomp $titleline;
 my @titles = split(/\t/, $titleline);
 
 while (<IN>)
  {
  chomp;
  #s/NA//g;
  my @temp = split/\t/;
  for (my $i = 0; $i <= $#temp; $i++)
   {
   $data{$temp[0]}{$titles[$i]} = $temp[$i];
   }
  }
 return %data; 
 }